
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * Copyright (c) 2018 STMicroelectronics International N.V. 
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of STMicroelectronics nor the names of other 
  *    contributors to this software may be used to endorse or promote products 
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this 
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for STMicroelectronics.
  * 5. Redistribution and use of this software other than as permitted under 
  *    this license is void and will automatically terminate your rights under 
  *    this license. 
  *
  * THIS SOFTWARE IS PROVIDED BY STMICROELECTRONICS AND CONTRIBUTORS "AS IS" 
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
  * SHALL STMICROELECTRONICS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f1xx_hal.h"
#include "usb_device.h"

/* USER CODE BEGIN Includes */
#define	TRACK		__NOP();
#define	BREAK		__NOP();


#define	TIM_LOGO					5							// Czas [s] wyswietlania logo
#define	TIM_DIAGNOSTIUC		10						// Czas [s] wyswietlanie ekranu diagnostycznego po resecie


//Przesuniecie FLASH dla BOOTLOADERA w kompilatorze, zmiana rozmiaru FLASH, ponadto
//	"stm32f103xb.h" definicja FLASH_BASE            0x08000000U
//	zmienic na taka jak w kompilatorze
#include <string.h>
#include <ctype.h>
#include "def.h"
#include "ansi.h"
#include "crc32.h"

enum {ANSI_BLACK, ANSI_RED, ANSI_GREEN, ANSI_YELLOW, ANSI_BLUE, ANSI_MAGNETA, ANSI_CYAN, ANSI_WHITE,  ANSI_INVERSE };
#include "graph.h"
#include "ssd1306_spi.h"


#include "usbd_cdc_if.h" // Plik bedacy interfejsem uzytkownika do kontrolera USB

#define LICZ_LICZ       		8        	// Liczba licznikw, pierwszy globalny
#define MAX_LEN_NAME_CNT    16

#define CNT_EDIT_MIN   			3       	// Min nr licznika ktory mozna modyfikowac z klawiatury i terminala
#define	CNT_RESET_MIN				1					// Min nr licznika, ktory mozna resetowac z terminala
#define CNT_EDIT_MAX    		4-2     	// Max nr licznika modyfikowanego z klawiatury

#define TIM_WR_FRAM         10       	// Co ile sekund zapisywac FRAM
#define	TIM_WR_FRAM_LAST		10				// Po jakim czasie od resetu pierwszy zapis po
#define	TIM_PRT_SAVE				1000			// Czas w ms wyswietlania (migania) naspisu SAVE


/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/
I2C_HandleTypeDef hi2c1;

IWDG_HandleTypeDef hiwdg;

RTC_HandleTypeDef hrtc;

SPI_HandleTypeDef hspi1;

TIM_HandleTypeDef htim3;

UART_HandleTypeDef huart2;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/
//todo: wyprobowac zapisFRAM w przerwaniach ale wtedy trzeba obie kopie zapisac za jednym razem

//--------- Konfiguracja uzywana takze przez FRAM ------//
typedef struct CfgFram
{ //----- Page 0 -----//
  u32 crc;            // crc musi byc pierwsza zmienna w strukturze
  u32 CntWrite;       // Liczba cykli zapisu EEprom
  //----- Page 1 -----//
  u32 CntRst;         // liczba resetow uC
  u32 recovered;      // Licznik liczby odzyskanych backup'ow

	u32 CntWdg;
	u32 CntBod;
	
	u32	baud;
  u32 CzasPracy[LICZ_LICZ];     					 	// Licznik czasu pracy (sekundy)
  char Txt[LICZ_LICZ][MAX_LEN_NAME_CNT];		// Komentarz licznikow
#if LICZ_LICZ < 8
  u8  cfg;
#elif LICZ_LICZ < 16
  u16 cfg;
#elif LICZ_LICZ < 32
  u32 cfg;
#else
#error "Za duzo licznikow"
#endif
#define	DZIEWICA_INIT					1
#define	DZIEWICA_RECOVERED		2
  u8 dziewica;
#define	CFG_TERM_TLO					1
	u8 cfgTerm;					// On/off tla wyswietlacza
  u32 eof;            // KONIEC PLIKU
} CfgFram;
struct CfgFram Liczniki;


//--------- Obszar nieinicjalizowany po resecie CPU ------//
//0x20000000    // poczatek RAM uC
//0x20001000    // przesuniety RAM dla kompilatora
//size zmniejszyc z 0x5000 na 0x4000
struct NOINIT
{
  u32 init;
  u32 rst;
  CfgFram cnt;
};
struct NOINIT nvram __attribute__((at(0x20000000)));
//const int x1 __attribute__((at(0x20000000))) = 10;

#define CFG_OK            	  0
#define RECORVED_NVRAM    	  1
#define RECORVED_FRAM1    	  2
#define RECORVED_FRAM2    	  4
#define RECORVED_CRASH    	  8
#define RECORVED_IIC_ERR	  128
#define CFG_ERR_IIC       	RECORVED_IIC_ERR
u8 Recovered, FL_EnBac2nvRam, FL_RunCnt = true;
uint32_t CrcRam(void);
u8 StI2cFramWr, StI2cFramRd;


//todo: bufor nalezalobu ograniczyc do 1000 a nadawanie USB zrobic "na raty"
#define	STR_LEN		1500
char str[STR_LEN];
u8 FL_BufStrErr, FL_Iwdg, FL_Bod;


u8 ReadFram(void);
void SetBaud( u32 baud );
char ReceivedUsart[8];
#define	RX_USAER_LEN			100
char RxUsartBuf[RX_USAER_LEN];
u8 volatile PtrWrUsart, PtrRdUsart;

uint8_t ReceivedUsbData[RX_USB_LEN]; // Tablica przechowujaca odebrane dane
uint16_t volatile ReceivedUsbDataFlag = false; // Flaga informujaca o odebraniu danych i ich dlugosci

const char nazwa[] = "Licznik Motogodzin";
const char nravt[] = "Licznik AVT";
const char nazwaLong[] = "Licznik Motogodzin v1.0";
const char nazwaFull[] = "Licznik Motogodzin v1.0\r\n"__DATE__" "__TIME__;
const char wersjaFull[] = "v1.0 rev1826\r\n"__DATE__" "__TIME__;
const char email[] = "es2@ep.com.pl";


const char txtOstrzezenie[] = "Ostrze"_z"enie";
const char txtAwaria[] = "Awaria";
const char txtBladKrytyczny[] = "B"_l""_a"d krytyczny";
const char txtAlarm[] = "Alarm";


void PrintOknoKom( char* belka, char* txt1, char* txt2 );

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_RTC_Init(void);
static void MX_I2C1_Init(void);
static void MX_IWDG_Init(void);
static void MX_SPI1_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_TIM3_Init(void);

void HAL_TIM_MspPostInit(TIM_HandleTypeDef *htim);
                                

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */


//------------------------------------------------------------
#define KEY_S1      1
#define KEY_S2      2
#define KEY_RST1    4
#define KEY_RST2    8

//------------------------------------------------------------
u8 volatile Key = 0, FL_KeyUnused = true;
u8 volatile FL_WrFram;
u16 volatile TimLedRun, TimLedOk, TimPushKey, TimSend, TimSof, TimOknoTestowe, TimPrtSave, TimRefreshLcd = TIM_DIAGNOSTIUC * 1000 + TIM_LOGO * 1000;
u32 volatile TimDemo;
u64 volatile TimSys;
//------------------------------------------------------------
void Irq1ms() {
	TimSys++;
  if ( TimLedRun ) TimLedRun--;
  if ( TimLedOk ) TimLedOk--;
  if ( TimRefreshLcd ) TimRefreshLcd--;
  if ( TimSend ) TimSend--;
  if ( TimSof ) TimSof--;
  if ( TimPrtSave ) TimPrtSave--;
	if ( TimOknoTestowe ) TimOknoTestowe--;
	if ( TimDemo ) TimDemo--;


    u8 static CzyRosnie=1, Duty=50;
    if ( !TimLedRun ) {
      TimLedRun = 10;

      if (Duty == 97) // Jezeli wypelnienie jest rowne 100 Zmien kierunek zliczania
        CzyRosnie = 0; //  w dol
      else if (Duty == 1) // Jezeli wypelnienie rowne 0
        CzyRosnie = 1; // Zmien kierunek zliczania w gore
      if (CzyRosnie) ++Duty; else --Duty;
			
      __HAL_TIM_SET_COMPARE(&htim3, TIM_CHANNEL_2, Duty);
      //      TIM3->CCR2 = Duty; // Wstawienie wyliczonej wartosci wypelnienia do
      // rejestru timera odpowiedzialnego za wypelnienie generowanego sygnalu PWM
    }

	
  u8 static TimKey;
  if ( TimKey ) TimKey--;
  else {
    TimKey = 30;

    u8 static prevK1 = true, prevK2 = true, prevR1 = true, prevR2 = true;
    if ( prevK1 == HAL_GPIO_ReadPin( Key1_GPIO_Port, Key1_Pin ) ) {
      if ( !prevK1 ) Key |= KEY_S1; else Key &= ~KEY_S1;
    }
    if ( prevK2 == HAL_GPIO_ReadPin( Key2_GPIO_Port, Key2_Pin ) ) {
      if ( !prevK2 ) Key |= KEY_S2; else Key &= ~KEY_S2;
    }
    if ( prevR1 == HAL_GPIO_ReadPin( Rst1_GPIO_Port, Rst1_Pin ) ) {
      if ( !prevR1 ) Key |= KEY_RST1; else Key &= ~KEY_RST1;
    }
    if ( prevR2 == HAL_GPIO_ReadPin( Rst2_GPIO_Port, Rst2_Pin ) ) {
      if ( !prevR2 ) Key |= KEY_RST2; else Key &= ~KEY_RST2;
    }
    prevK1 = HAL_GPIO_ReadPin( Key1_GPIO_Port, Key1_Pin );
    prevK2 = HAL_GPIO_ReadPin( Key2_GPIO_Port, Key2_Pin );
    prevR1 = HAL_GPIO_ReadPin( Rst1_GPIO_Port, Rst1_Pin );
    prevR2 = HAL_GPIO_ReadPin( Rst2_GPIO_Port, Rst2_Pin );
    if ( Key & 3 ) FL_KeyUnused = false;
  }
  if ( Key & 15 ) TimPushKey++; else TimPushKey = 0;   // Wykrywanie dlugie naciskania ktoregokolwiek klawisza


  u16 static d100ms = 1, d60s = TIM_WR_FRAM_LAST;
  if ( FL_RunCnt ) {          // Sprawdzanie w tym miejscu nie spowoduje niewykonania operacji tylko ja opozni gdy flaga bedzie ustawiona
    if ( !d100ms-- ) {
      d100ms = 1000;
      if ( !d60s-- ) {
        d60s = TIM_WR_FRAM;
        FL_WrFram = true;
      }

      Liczniki.cfg |= 1;      // Pierwszy licznik zawsze wlaczony
      // drugi i trzeci zalezy od stanu wejsc En1 i En2
      if ( HAL_GPIO_ReadPin( EN1_GPIO_Port, EN1_Pin ) ) Liczniki.cfg &= ~2; else Liczniki.cfg |= 2;
      if ( HAL_GPIO_ReadPin( EN2_GPIO_Port, EN2_Pin ) ) Liczniki.cfg &= ~4; else Liczniki.cfg |= 4;
      for (u8 x = 0; x < LICZ_LICZ; x++) {
        if ( Liczniki.cfg & 1 << x) Liczniki.CzasPracy[x]++;
      }

      if ( FL_EnBac2nvRam ) {
        Liczniki.crc = CrcRam();
        nvram.cnt = Liczniki;     // Struktura z RAM do NvRam
      }
    }
  }
}


//------------------------------------------------------------
u64 ReadTimSys()
{
	__disable_irq();
	u64 tim = TimSys;
	__enable_irq();

	return tim;
}


//------------------------------------------------------------
void PrintError() {
  //todo: miganie LED, komunikaty na LCD jak w ster podajnika
  HAL_GPIO_WritePin( LED_ERR_GPIO_Port, LED_ERR_Pin, GPIO_PIN_RESET );
  while ( true ) {
    IWDG->KR = 0xaaaa;      //przeladowanie (reload) IWDG
  }
}


//------------------------------------------------------------
#define EE_IIC_ADR    0xa0
//#define EE_IIC_SIZE   8192    // 24c64
#define EE_IIC_SIZE   2048      // 24c16
//#define EE_IIC_PAGE     8


#define IIC_MAX_ERR     5
uint16_t IIcReadErr = 0, IIcCrcErr = 0;


//------------------------------------------------------------
uint32_t Crc( u8* p, u16 len )
{
	return CalculateCrc32( p, len, len );	// Inicjalizacja wartoscia LEN a nie 0xFFFF

	
	//  uint32_t crc = len;       // Init CRC


	// liczba bajtow od pierwszego bajtu za CRC do ostatniego bajtu struktury
/*
  for (uint8_t x = 0; x < len; x++) {
    crc += *p++;
  }
  return crc ^ 0xFFFFFFFF;
*/
/*
  for (uint8_t x = 0; x < len; x++) {
    crc = xcrc32( crc );
  }
  return crc ^ 0xFFFFFFFF;
*/
}


//------------------------------------------------------------
uint32_t CrcRam()
{
  uint8_t* p;


  p = (uint8_t*)&Liczniki;     // Wskaznik na strukrure
  p += sizeof(Liczniki.crc);   // Zwiekszamy o zmienna CRC


  // liczba bajtow od pierwszego bajtu za CRC do ostatniego bajtu struktury
	return( Crc( p, sizeof(Liczniki)-sizeof(Liczniki.crc) ) );
}


//------------------------------------------------------------
uint32_t CrcNvRam()
{
  uint8_t* p;


  p = (uint8_t*)&nvram.cnt;     // Wskaznik na strukrure
  p += sizeof(nvram.cnt.crc);   // Zwiekszamy o zmienna CRC


  // liczba bajtow od pierwszego bajtu za CRC do ostatniego bajtu struktury
	return( Crc( p, sizeof(nvram.cnt)-sizeof(nvram.cnt.crc) ) );
}


//------------------------------------------------------------
// przy 1MHz zapis xxx bajtow 3,3ms
//------------------------------------------------------------
uint8_t WrFram()
{
  static struct CfgFram BakCnt;     // Static rezerwuje RAM globalnie, nie trzeba zwieszac sterty jak bedzie duza struktura

  Liczniki.eof = 0xAA55FFFF;
  Liczniki.CntWrite++;
  __disable_irq();
  Liczniki.crc = CrcRam();
  BakCnt = Liczniki;                // Kopia struktury w RAM
  __enable_irq();


  //----- Zapis kopii do Fram
#if EE_IIC_SIZE <= 2048
  StI2cFramWr = HAL_I2C_Mem_Write(&hi2c1, EE_IIC_ADR, 0, 1, ((uint8_t*)&BakCnt), sizeof(BakCnt), 20);
	//todo: jesli rozmiar wiekszy niz 256 baktow, adresowanie takze w adresie IIC
#else
  StI2cFramWr = HAL_I2C_Mem_Write(&hi2c1, EE_IIC_ADR, 0, 2, ((uint8_t*)&BakCnt), sizeof(BakCnt), 20);
#endif
  if ( StI2cFramWr != HAL_OK ) return CFG_ERR_IIC;


#if EE_IIC_SIZE <= 2048
  StI2cFramWr = HAL_I2C_Mem_Write(&hi2c1, EE_IIC_ADR, sizeof(Liczniki), 1, ((uint8_t*)&BakCnt), sizeof(BakCnt), 20);
#else
  StI2cFramWr = HAL_I2C_Mem_Write(&hi2c1, EE_IIC_ADR, sizeof(Liczniki), 2, ((uint8_t*)&BakCnt), sizeof(BakCnt), 20);
#endif
  if ( StI2cFramWr != HAL_OK ) return CFG_ERR_IIC;


  TRACK;
  return CFG_OK;
}


//------------------------------------------------------------
uint8_t WriteFram()
{
  Liczniki.dziewica = 0;
  return ( WrFram() );
}


//------------------------------------------------------------
uint8_t InitFram()
{
  uint8_t stat;


  FL_RunCnt = false;    // Zatrzymanie zliczania

  Liczniki.cfg = 0;
  for (u8 x = 0; x < LICZ_LICZ; x++) {
    Liczniki.CzasPracy[x] = 0;
    sprintf( str, "Cnt %x", x );
    strncpy( Liczniki.Txt[x], str, MAX_LEN_NAME_CNT );
  }
  strncpy( Liczniki.Txt[0], "Global", MAX_LEN_NAME_CNT );
  strncpy( Liczniki.Txt[1], "AVT Korporacja", MAX_LEN_NAME_CNT );
  strncpy( Liczniki.Txt[2], email, MAX_LEN_NAME_CNT );


	Liczniki.baud = 115200;
	Liczniki.cfgTerm = CFG_TERM_TLO;

  Liczniki.CntRst = 0;
  Liczniki.recovered = 0;
  Liczniki.dziewica = DZIEWICA_INIT;

  FL_RunCnt = true;


  stat = WrFram();


  return stat;
}


//------------------------------------------------------------
u8 ReadFram()
{
  uint8_t l = 0;


  TRACK;
  do {
#if EE_IIC_SIZE <= 2048
    StI2cFramRd = HAL_I2C_Mem_Read(&hi2c1, EE_IIC_ADR, 0, 1, (uint8_t*)&Liczniki, sizeof(Liczniki), 10);
#else
    StI2cFramRd = HAL_I2C_Mem_Read(&hi2c1, EE_IIC_ADR, 0, 2, (uint8_t*)&Liczniki, sizeof(Liczniki), 10);
#endif
    l++;
  } while ( StI2cFramRd != HAL_OK && l < IIC_MAX_ERR ); // piec odczytow jesli blad

  if ( l > 1 ) {
    IIcReadErr += l - 1;      // Zwieksz licznik blednych odczytow
	if( l >= IIC_MAX_ERR ){
		Recovered |= RECORVED_IIC_ERR;
    return RECORVED_IIC_ERR;       // Blad IIC
}
  }


  __disable_irq();
  u32 crc = CrcRam();
  __enable_irq();
  if ( Liczniki.crc != crc ) {                // Jesli bledne CRC to odczytaj z drugiej kopii
#if EE_IIC_SIZE <= 2048
    StI2cFramRd = HAL_I2C_Mem_Read(&hi2c1, EE_IIC_ADR, sizeof(Liczniki), 1, (uint8_t*)&Liczniki, sizeof(Liczniki), 10);
#else
    StI2cFramRd = HAL_I2C_Mem_Read(&hi2c1, EE_IIC_ADR, sizeof(Liczniki), 2, (uint8_t*)&Liczniki, sizeof(Liczniki), 10);
#endif
    FL_RunCnt = true;


    __disable_irq();
    u32 crc = CrcRam();
    __enable_irq();
    if ( Liczniki.crc != crc ) {              // Jesli bledna druga kopia to init pamieci
      IIcCrcErr++;
      InitFram();
      Recovered |= RECORVED_CRASH;
      return ( RECORVED_CRASH );
    }
    Liczniki.recovered++;
    Liczniki.dziewica = DZIEWICA_RECOVERED;
    WrFram();                       // Zapis naprawionej kopii
    Recovered |= RECORVED_FRAM2;
    return RECORVED_FRAM2;
  }


  Recovered |= RECORVED_FRAM1;
  TRACK;
  return RECORVED_FRAM1;
}


//------------------------------------------------------------
void SendData( char* data, u16 len )
{
	CDC_Transmit_FS( (uint8_t*)data, len );
	HAL_UART_Transmit_IT(&huart2, (uint8_t*)data, len ); // Rozpoczecie nadawania danych z wykorzystaniem przerwan
}


//------------------------------------------------------------
u8 AdresEeprom = 0;
//------------------------------------------------------------
void ObslugaLedtAlertow()
{
  if ( Liczniki.dziewica ) HAL_GPIO_WritePin( LED_ALERT_GPIO_Port, LED_ALERT_Pin, GPIO_PIN_RESET );
		else HAL_GPIO_WritePin( LED_ALERT_GPIO_Port, LED_ALERT_Pin, GPIO_PIN_SET );

	if ( !AdresEeprom ) {
    HAL_GPIO_WritePin( LED_ALERT_GPIO_Port, LED_ALERT_Pin, GPIO_PIN_RESET );
    HAL_GPIO_WritePin( LED_ERR_GPIO_Port, LED_ERR_Pin, GPIO_PIN_RESET );
  }
}


//------------------------------------------------------------
void ReanimacjaI2C()
{
#define PORT_I2C		GPIOB
#define	PIN_SCL			GPIO_PIN_6
#define	PIN_SDA			GPIO_PIN_7


		HAL_GPIO_WritePin( GPIOB, PIN_SDA, GPIO_PIN_SET );	// SDA=H
		HAL_GPIO_WritePin( GPIOB, PIN_SCL, GPIO_PIN_SET );	// SCL=H
		HAL_Delay(1);

		//Configure GPIO Output pins : PB6 - SCL, PB7 - SDA
		GPIO_InitTypeDef GPIO_InitStruct;
		GPIO_InitStruct.Pin = PIN_SCL | PIN_SDA;
		GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
		GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
		HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

		HAL_GPIO_WritePin( GPIOB, PIN_SDA, GPIO_PIN_RESET );	// SDA=L
		HAL_Delay(1);
		HAL_GPIO_WritePin( GPIOB, PIN_SCL, GPIO_PIN_RESET );	// SCL=L
		for(u8 x=0; x<9; x++){
			HAL_Delay(1);
			HAL_GPIO_WritePin( GPIOB, PIN_SCL, GPIO_PIN_SET );	// SCL=H
			HAL_Delay(1);
			HAL_GPIO_WritePin( GPIOB, PIN_SCL, GPIO_PIN_RESET );	// SCL=L
			}
		//----- Stop IIC -----//
		HAL_Delay(1);
		HAL_GPIO_WritePin( GPIOB, PIN_SCL, GPIO_PIN_SET );	// SCL=H
		HAL_Delay(1);
		HAL_GPIO_WritePin( GPIOB, PIN_SDA, GPIO_PIN_SET );	// SDA=H
}


//------------------------------------------------------------
void LosujCzasDemo()
{
	TimDemo = (rand()% 10 + 10 ) * 60 * 1000;		// Co 10..20 minut
	TRACK;
}


//------------------------------------------------------------
void StringDemo()
{
      strcpy( str, nazwaFull); strcat( str, "\r\n"); strcat( str, email ); strcat( str, "\r\n");
      SendData( str, strlen(str) );
}


//------------------------------------------------------------
void SkanujIIC()
{
  char buffer_str[30];
  uint8_t Data[4];
  uint16_t pozX = 0;

  ssd1306_clearDisplay();

  ssd1306_PrintString( 0, 0, (char*)nazwaLong, ~ANSI_WHITE);
  sprintf( str, "%s %s", __DATE__, __TIME__ );
  ssd1306_PrintString( 0, 8, str, ANSI_WHITE);
  ssd1306_display(); // ssd1306_invertDisplay( true );

  ssd1306_PrintString( 0, 16, "Skanowanie I2C...", ANSI_WHITE);
  ssd1306_display();


  AdresEeprom = 0;
  for (uint8_t adr = 0; adr < 128; adr++)
  {
    uint8_t stat = HAL_I2C_Mem_Read(&hi2c1, adr << 1, 0, 1, Data, 1, 2);
    if ( stat == HAL_OK )
    {
      AdresEeprom = adr;
      sprintf(buffer_str, "%02x ", adr << 1);
      pozX = ssd1306_PrintString( pozX, 24, buffer_str, ANSI_WHITE);
      ssd1306_display();
    }
  }
	if( pozX > SSD1306_LCDWIDTH ) pozX = SSD1306_LCDWIDTH - 8;
  ssd1306_PrintString( pozX, 24, "End", ~ANSI_WHITE);
  ssd1306_display();


	//----- Obsluga licznikow -----//
  if ( sizeof(Liczniki) * 2 > EE_IIC_SIZE ) {
    sprintf(buffer_str, "EEprom %d Bajt"_o"w.", sizeof(Liczniki) * 2 );
    PrintOknoKom(  (char*)txtBladKrytyczny, buffer_str, " " );
    ssd1306_display();
    while ( 1 )IWDG->KR = 0xaaaa;     //przeladowanie (reload) IWDG
  }


  FL_RunCnt = false;      // Zatrzymanie zliczania
  u8 stat_cfg = ReadFram();
	SetBaud( Liczniki.baud );
  FL_RunCnt = true;
  if ( stat_cfg == RECORVED_IIC_ERR ) {
    PrintOknoKom(  (char*)txtBladKrytyczny, "Awaria I2C", "lub pami"_e"ci Fram" );
    ssd1306_display();
  }
  else {
    Liczniki.CntRst++;
		if( FL_Iwdg )Liczniki.CntWdg++;
		if( FL_Bod )Liczniki.CntBod++;
    WrFram();
    FL_RunCnt = false;    // Zatrzymanie zliczania
    ReadFram();
		SetBaud( Liczniki.baud );
    FL_RunCnt = true;
  }
  if ( nvram.cnt.crc == CrcNvRam() ) {  // Jesli CRC w NvRam ok
    __disable_irq();
    Liczniki = nvram.cnt;               // Struktura z NvRam do RAM
    __enable_irq();
    Recovered |= RECORVED_NVRAM;
    TRACK;
  }
  if ( AdresEeprom )FL_EnBac2nvRam = true; // Pozwolenie na wykonywanie kopii w NvRam
	srand( Liczniki.crc );
	LosujCzasDemo();


  u8 pozY = 32;
  sprintf(buffer_str, "[%d]%d ei%d ec%d rec%d", sizeof(Liczniki), Recovered, IIcReadErr, IIcCrcErr, (int)Liczniki.recovered );
  pozX = ssd1306_PrintString( 0, pozY, buffer_str, ANSI_WHITE);


  pozY = 40;
  sprintf(buffer_str, "I%d B%d R%d W%d", (int)Liczniki.CntWdg, (int)Liczniki.CntBod, (int)Liczniki.CntRst, (int)Liczniki.CntWrite  );
  pozX = ssd1306_PrintString( 0, pozY, (char*)buffer_str, ANSI_WHITE);
  if ( Liczniki.dziewica == DZIEWICA_INIT ) pozX = ssd1306_PrintString( pozX, pozY, "INIT", ~ANSI_WHITE);
  else if ( Liczniki.dziewica == DZIEWICA_RECOVERED ) pozX = ssd1306_PrintString( pozX, pozY, "RECOVERED", ~ANSI_WHITE);
  ObslugaLedtAlertow();


  pozY = 48;
  sprintf(buffer_str, "USART %dbaud ", (int)Liczniki.baud );
  pozX = ssd1306_PrintString( 0, pozY, (char*)buffer_str, ANSI_WHITE);


  /*
    uint32_t id_cpu[3];
    HAL_GetUID(id_cpu);

    //    void HAL_GetUID(uint32_t *UID)
    //    {
    //    UID[0] = (uint32_t)(READ_REG(*((uint32_t *)UID_BASE)));
    //    UID[1] = (uint32_t)(READ_REG(*((uint32_t *)(UID_BASE + 4U))));
    //    UID[2] = (uint32_t)(READ_REG(*((uint32_t *)(UID_BASE + 8U))));
    //    }
    uint32_t crcid = id_cpu[0] ^ id_cpu[1] ^ id_cpu[2];

    //  sprintf(buffer_str, "id= %x:%x", id_cpu[2], id_cpu[1] );
    //  st7565_PrintString(&cfg_lcd, 0, 40, buffer_str, ANSI_WHITE);
    //  sprintf(buffer_str, " :%x - %x", id_cpu[0], crcid );
    ////  sprintf(buffer_str, " :%x", id_cpu[0] );
    //  st7565_PrintString(&cfg_lcd, 0, 48, buffer_str, ANSI_WHITE);
    sprintf(buffer_str, "%s %s", __DATE__, __TIME__ );
    st7565_PrintString(&cfg_lcd, 0, 40, buffer_str, ANSI_WHITE);
    sprintf(buffer_str, "ID=%x", crcid );
    st7565_PrintString(&cfg_lcd, 0, 48, buffer_str, ANSI_WHITE);


    //  sprintf(buffer_str, "Str '%s'", Cfg.txt );
    //  pozX = st7565_PrintString(&cfg_lcd, 0, 48, buffer_str, ANSI_WHITE);
  */


  /*
    //============== zabezpieczenia =================//
    if ( crcid != 0x73a616c && crcid != 0x7636174 )
    {
      sprintf( txt, "ID=%x", crcid );
      PrintOknoKom( &cfg_lcd, (char*)txtBladKrytyczny, "B"_l""_a"d FLASH", txt );
      st7565_Draw( &cfg_lcd );
      HAL_GPIO_WritePin( LED_R_GPIO_Port, LED_R_Pin, GPIO_PIN_RESET );
      while ( true) IWDG->KR = 0xaaaa;    //przeladowanie (reload) IWDG
    }
    if( Cfg.CntWrite > 10000 || Cfg.CzasPracy > 2592000 ) // 30 dni
    {
      PrintOknoKom( &cfg_lcd, (char*)txtBladKrytyczny, "B"_l""_a"d EEprom", "" );
      st7565_Draw( &cfg_lcd );
      HAL_GPIO_WritePin( LED_R_GPIO_Port, LED_R_Pin, GPIO_PIN_RESET );
      while ( true) IWDG->KR = 0xaaaa;    //przeladowanie (reload) IWDG
    }
  */


  ssd1306_display();


  if ( !AdresEeprom ) {
    PrintOknoKom( "Awaria", "Nie wykryto FRAM", "lub b"_l""_a"d I2C" );
    ssd1306_display();
		//----- Ozywianie I2C (zablokowany slave) -----//
		HAL_I2C_DeInit(&hi2c1);
		ReanimacjaI2C();
    while ( true ) { __NOP(); };    // Jesli brak eeprom to reset CPU przez WDG
  }
}


//------------------------------------------------------------
void SetBaud( u32 baud )
{
		huart2.Instance = USART2;
		huart2.Init.BaudRate = baud;
		huart2.Init.WordLength = UART_WORDLENGTH_8B;
		huart2.Init.StopBits = UART_STOPBITS_1;
		huart2.Init.Parity = UART_PARITY_NONE;
		huart2.Init.Mode = UART_MODE_TX_RX;
		huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
		huart2.Init.OverSampling = UART_OVERSAMPLING_16;
		if (HAL_UART_Init(&huart2) != HAL_OK){
			_Error_Handler(__FILE__, __LINE__);
			}

		HAL_UART_Receive_IT(&huart2, (uint8_t*)ReceivedUsart, 1);	 // Wlaczenie nasluchiwania
}


//------------------------------------------------------------
char o_belka[30], o_wiersz_1[30], o_wiersz_2[30];
//------------------------------------------------------------
void InterpretujCmd( char znak )
{
  char const StrErrVal[] = "@Ev\r\n";
  char const StrErrUnkown[] = "@Eu\r\n";


#define MAX_LEN_CMD     50
  char static p = 0, cmd[MAX_LEN_CMD];
  int cnt;
  char static buffer_str[100];
	int baud;


  cmd[p] = znak;
  if ( p < MAX_LEN_CMD ) p++;



  if ( znak == '\r' || znak == '\n' ) {      // Interpretacja komendy (CR LF)
    if ( strncmp( cmd, "ati", 3 ) == 0 ) StringDemo();
    else if ( strncmp( cmd, "rst", 3 ) == 0 ) {
      strcpy( str, "Restart...\r\n");
      SendData( str, strlen(str) );
			HAL_Delay(10);
			HAL_GPIO_WritePin( USB_PU_GPIO_Port, USB_PU_Pin, GPIO_PIN_RESET );
			USBD_DeInit( &hUsbDeviceFS );
			HAL_Delay(100);
			while( true );
			}
    else if ( cmd[0] == '@' ) {
      switch ( tolower(cmd[1]) )
      {
				case 'o':														// Okno testowe na LCD: @o1 belka	@o2 wiers_1	@o3 wiersz_2
          sscanf( cmd + 2, "%d", &cnt );
          if ( cnt == 1 ) { sscanf( cmd + 4, "%s", o_belka ); TimOknoTestowe = 30000; }
          if ( cnt == 2 ) { sscanf( cmd + 4, "%s", o_wiersz_1 ); TimOknoTestowe = 30000; }
          if ( cnt == 3 ) { sscanf( cmd + 4, "%s", o_wiersz_2 ); TimOknoTestowe = 30000; }
          else {
            SendData( (char*)StrErrVal, strlen(StrErrVal) );
          }
          sprintf( str, "o:\r\n%s\r\n%s\r\n%s\r\n", o_belka, o_wiersz_1, o_wiersz_2 );
          SendData( str, strlen(str) );
					break;
				case '?':
          strcpy( str, "?:\r\n" );
          sprintf(buffer_str, "[%d]%d ei%d ec%d rec%d", sizeof(Liczniki), Recovered, IIcReadErr, IIcCrcErr, (int)Liczniki.recovered );
          strcat( str, "\r\n" ); strcat( str, buffer_str );
          sprintf(buffer_str, "Wr=%d Rst=%d", (int)Liczniki.CntWrite, (int)Liczniki.CntRst );
          strcat( str, "\r\n" ); strcat( str, buffer_str );
          sprintf(buffer_str, "Adr=$%x", (int)AdresEeprom );
          strcat( str, "\r\n" ); strcat( str, buffer_str );
          sprintf(buffer_str, "Baud=%d Init=%d Crc=$%X", (int)Liczniki.baud, Liczniki.dziewica, (int)Liczniki.crc & 0xFFFF );
          strcat( str, "\r\n" ); strcat( str, buffer_str );
	        sprintf(buffer_str, "nvram.rst=%d Bod=%d Iwdg=%d", (int)nvram.rst, FL_Bod, FL_Iwdg  );
          strcat( str, "\r\n" ); strcat( str, buffer_str );
	        sprintf(buffer_str, "TimDemo=%ds %dmin", (int)(TimDemo/1000), (int)(TimDemo/(1000*60))  );
          strcat( str, "\r\n" ); strcat( str, buffer_str );
					strcat( str, "\r\n" );
          SendData( str, strlen(str) );
          break;
        case 's':                         // Start licznika: @sx
          sscanf( cmd + 2, "%d", &cnt );
          if ( cnt >= CNT_EDIT_MIN && cnt < LICZ_LICZ ) {
            Liczniki.cfg |= (1 << cnt);
            sprintf( str, "@s:%d\r\n", cnt );
            SendData( str, strlen(str) );
            WriteFram();
          }
          else {
            SendData( (char*)StrErrVal, strlen(StrErrVal) );
          }
          break;
        case 'e':                         // Stop licznika: @ex
          sscanf( cmd + 2, "%d", &cnt );
          if ( cnt >= CNT_EDIT_MIN && cnt < LICZ_LICZ ) {
            Liczniki.cfg &= ~(1 << cnt);
            sprintf( str, "@e:%d\r\n", cnt );
            SendData( str, strlen(str) );
            WriteFram();
          }
          else {
            SendData( (char*)StrErrVal, strlen(StrErrVal) );
          }
          break;
	     case 'i':                         // Odczyt ramek CSV
					sprintf( str, "i:%d\r\n;cnt;a;d;gg;mm;ss;txt;s\r\n", LICZ_LICZ );
					for (u8 x = 0; x < LICZ_LICZ; x++) {
						float czas = Liczniki.CzasPracy[x];
						u32 min = Liczniki.CzasPracy[x] / 60;
						u32 godz = Liczniki.CzasPracy[x] / 3600 ;
						u32 dni = Liczniki.CzasPracy[x] / 86400 ;
						char txt[100], aktywny = '-';
						if ( Liczniki.cfg & (1 << x) ) aktywny = '*';
						sprintf( txt, ";%d;%c;%d;%d:%02d:%02d;%s;%d\r\n", x, aktywny, (int)dni, (int)(godz % 24), (int)(min % 60), (int)(Liczniki.CzasPracy[x] % 60), Liczniki.Txt[x], (int)Liczniki.CzasPracy[x] );
						strcat( str, txt );
						}
          SendData( str, strlen(str) );
					break;
				 //todo: moze resetowanie tylko gdy licznik zatrzymany? wtedy, przy probie resetu, gdy liczy komunikat bledu
				case 'r':                         // Resetowanie licznika: @rx
          sscanf( cmd + 2, "%d", &cnt );
          if ( cnt >= CNT_RESET_MIN && cnt < LICZ_LICZ ) {
            Liczniki.cfg &= ~(1 << cnt);  // stop
            Liczniki.CzasPracy[cnt] = 0;
            sprintf( str, "@r:%d\r\n", cnt );
            SendData( str, strlen(str) );
            WriteFram();
          }
          else {
            SendData( (char*)StrErrVal, strlen(StrErrVal) );
          }
          break;
        case 't':                         // Zmiana kometarza: @tx,txt
          sscanf( cmd + 2, "%d", &cnt );
          if ( cnt > 0 && cnt < LICZ_LICZ ) {
            strncpy( Liczniki.Txt[cnt], cmd + 4, MAX_LEN_NAME_CNT );
            Liczniki.Txt[cnt][MAX_LEN_NAME_CNT - 1] = 0; // znak konca tekstu gdyby byl dluzszy od MAX_LEN_NAME_CNT
            for (u8 x = 0; x < MAX_LEN_NAME_CNT; x++) { // zamiana CR/LF na koniec tekstu
              if ( Liczniki.Txt[cnt][x] == '\n' || Liczniki.Txt[cnt][x] == '\r' ) Liczniki.Txt[cnt][x] = 0;
            }
            WriteFram();
            sprintf( str, "@t%d:%s\r\n", cnt, Liczniki.Txt[cnt] );
            SendData( str, strlen(str) );
          }
          else {
            SendData( (char*)StrErrVal, strlen(StrErrVal) );
          }
          break;
        case 'n':
          StringDemo();
          break;
        case 'v':
          strcpy( str, wersjaFull); strcat( str, "\r\n");
          SendData( str, strlen(str) );
          break;
				case 'b':
					sscanf( cmd + 2, "%d", &baud );
					Liczniki.baud = baud;
					SetBaud( baud );
					WriteFram();
					sprintf( str, "@b:%d\r\n", (int)Liczniki.baud );
          SendData( str, strlen(str) );
					break;
				case 'c':
					sscanf( cmd + 2, "%d", &cnt );		// Konfiguracja terminala: @cx
					if ( cnt ) Liczniki.cfgTerm |= CFG_TERM_TLO; else Liczniki.cfgTerm &= ~CFG_TERM_TLO;
          sprintf( str, "@c:%x\r\n", Liczniki.cfgTerm );
	        SendData( str, strlen(str) );
					WriteFram();
					break;
        default:
          SendData( (char*)StrErrUnkown, strlen(StrErrUnkown) );
      }
    }
		for( char x=0; x<MAX_LEN_CMD; x++) cmd[x]=0;
    p = 0;
  }
}


//------------------------------------------------------------
void PrintOknoKom( char* belka, char* txt1, char* txt2 )
{
  int const pozY = 64 / 2 - 4 * 8 / 2;
#define SZER_CZCIONKI   6

  int szer = strlen(belka);
  if ( strlen(txt1) > szer ) szer = strlen(txt1);
  if ( strlen(txt2) > szer ) szer = strlen(txt2);
  szer += 2;
  szer *= SZER_CZCIONKI;

  int pozX = 128 / 2 - szer / 2;
  ssd1306_Box( pozX - 4, pozY, 4 * 8, szer, ANSI_BLACK, true ); // Czyszczenie okna
  ssd1306_Box( pozX - 4, pozY, 4 * 8, szer, ANSI_WHITE, false ); // Ramka
  ssd1306_Box( pozX - 4, pozY, 8, szer, ANSI_WHITE, true );     // Belka
  ssd1306_Box( pozX - 4 + szer - 5, pozY + 2, 4, 4, ANSI_BLACK, true ); // Kwadracik
  ssd1306_LineV( pozX - 4 + szer - 8, pozY, 8, ANSI_BLACK );    // kreseczka

  ssd1306_PrintString( pozX + szer / 2 - strlen(belka)*SZER_CZCIONKI / 2, pozY, belka, ~ANSI_WHITE );

  ssd1306_PrintString( pozX + szer / 2 - strlen(txt1)*SZER_CZCIONKI / 2, pozY + 8 + 4, txt1, ANSI_WHITE );
  ssd1306_PrintString( pozX + szer / 2 - strlen(txt2)*SZER_CZCIONKI / 2, pozY + 16 + 4, txt2, ANSI_WHITE );
}


//------------------------------------------------------------
void AwariaRCC()
{
  PrintOknoKom( (char*)txtBladKrytyczny, "Uszkodzenie", "generatora RCC" );
  ssd1306_display();
}


//------------------------------------------------------------


void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart) {

	RxUsartBuf[PtrWrUsart++] = ReceivedUsart[0];
	if( PtrWrUsart >= RX_USAER_LEN ) PtrWrUsart = 0;		// Bufor kolowy


	HAL_UART_Receive_IT(&huart2, (uint8_t*)ReceivedUsart, 1);	 // Wlaczenie nasluchiwania
}


//------------------------------------------------------------
int GetUsart()
{
int znak;


	if( PtrWrUsart != PtrRdUsart ){
			znak = RxUsartBuf[PtrRdUsart++];
			if( PtrRdUsart >= RX_USAER_LEN ) PtrRdUsart = 0;		// Bufor kolowy
		}
	else znak = 0xFFFF;


	return znak;
}


//------------------------------------------------------------
#define Page_127        0x0801FC00				// 128k Flash
//------------------------------------------------------------
void FLASH_read(uint16_t* data, uint16_t size)
{
	uint16_t i;
	for(i=0; i<size; i++){
		data[i] = *(__IO uint16_t*)(Page_127 + i*2);
	}

}


//------------------------------------------------------------
//----- dla 103 -----///
void FLASH_write(uint16_t* data, uint16_t size)
{
//0. Wyczyscic cala strone (jesli wyczyszczona, to pomijamy)
//1. Odblokowanie zapisu(dwa wpisy do FLASH->KEYR)
//2. Upewnienie sie, ze flash nie jest zajety  rejestr SR
//3. Wlaczenie zapisu do flasha  FLASH_CR_PG_bb = 1
//4. Dokonanie zapisu  normalny zapis jak do zmiennej w RAM, tyle ze rozmiar musi byc 16-bit
//5. Poczekac na zakonczenie  rejestr SR
//6. Wylaczenie zapisu do flash  FLASH_CR_PG_bb = 0
//7. Zablokowanie mozliwosci zpaisu  FLASH_CR_LOCK_bb = 1

	uint16_t i;
	FLASH->KEYR = FLASH_KEY1;
	FLASH->KEYR = FLASH_KEY2;
	//FLASH_Erase Page
	while((FLASH->SR&FLASH_SR_BSY));
	FLASH->CR |= FLASH_CR_PER; //Page Erase Set
	FLASH->AR = Page_127; //Page Address
	FLASH->CR |= FLASH_CR_STRT; //Start Page Erase
	while((FLASH->SR&FLASH_SR_BSY));
	FLASH->CR &= ~FLASH_CR_PER; //Page Erase Clear
	//FLASH_Program HalfWord
	FLASH->CR |= FLASH_CR_PG;
	for(i=0; i<size; i++){
		while((FLASH->SR&FLASH_SR_BSY));
		*(__IO uint16_t*)(Page_127 + i*2) = data[i];
	}
	FLASH->CR &= ~FLASH_CR_PG;
	FLASH->CR |= FLASH_CR_LOCK;



uint32_t static volatile Address = 0x0801F800;
//uint8_t static userConfig[64] __attribute__((at(0x0801F800)));
HAL_FLASH_Unlock();
//	HAL_FLASHEx_Erase(&EraseInitStruct, &SectorError)

//HAL_FLASH_Program(FLASH_TYPEPROGRAM_WORD, (uint32_t)&userConfig[0], 0x42);
if( HAL_FLASH_Program(FLASH_TYPEPROGRAM_HALFWORD, Address, 0x42434445)  == HAL_OK) {
      Address = Address + 4;
    }
    else {
      // Error occurred while writing data in Flash memory.
      //   User can add here some code to deal with this error
__NOP();
        //  FLASH_ErrorTypeDef errorcode = HAL_FLASH_GetError();
      _Error_Handler(__FILE__, __LINE__);
    }
HAL_FLASH_Lock();


}


//------------------------------------------------------------
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  *
  * @retval None
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
  //----- Zatrzymuj IWDG podczas debugowania -----//
  __HAL_DBGMCU_FREEZE_IWDG(); // DBGMCU -> APB1FZ |= DBGMCU_APB1_FZ_DBG_IWDG_STOP;
  //----- Nie rozlaczaj z debugerem po usypieniu
  DBGMCU->CR |= DBGMCU_CR_DBG_SLEEP;   //


  //----- Flagi resetu -----//
  if ( RCC->CSR & (1 << 29) ) FL_Iwdg = true; // RCC->CSR & RCC_FLAG_IWDGRST ???
  if ( RCC->CSR & (1 << 27) ) FL_Bod = true;
  __HAL_RCC_CLEAR_RESET_FLAGS();  // RCC->CSR |= RCC_CSR_RMVF;    // Kasuje flagi


#define U32INIT   0xAAbbCCdd
  nvram.rst++;
  if ( nvram.init != U32INIT )
  {
    nvram.init = U32INIT;
    FL_Bod = 0;                   // Pierwsze uruchomienie ignorujemy flage BOD
    nvram.rst = 0;
  }


	str[STR_LEN-1] = '@'; str[STR_LEN-2] = '#'; 
	
	
	//----- Opoznienia dla wystartowania USB
	for(u32 x=0; x<500000; x++) __NOP();
	IWDG->KR = 0xaaaa;      //przeladowanie (reload) IWDG


  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_RTC_Init();
  MX_USB_DEVICE_Init();
  MX_I2C1_Init();
  MX_IWDG_Init();
  MX_SPI1_Init();
  MX_USART2_UART_Init();
  MX_TIM3_Init();
  /* USER CODE BEGIN 2 */


	//----- Led RUN wyjsciem PP
  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Pin = LED_RUN_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

	if ( HAL_TIM_PWM_Start_IT(&htim3, TIM_CHANNEL_2) == HAL_OK ) { // Uruchamia generowanie PWM z przerwaniami
    //  HAL_TIMEx_PWMN_Start(&htim3, TIM_CHANNEL_2);
    TRACK;
  }
//	HAL_TIM_OC_Start_IT(&htim3, TIM_CHANNEL_2);		// Przerwania od porownania tmera
	HAL_TIM_Base_Start_IT(&htim3);	// Przerwanie od aktualizacji (przepelnieniu) licznika
	

	ssd1306_init();
	u8 pozY = 0;
  ssd1306_PrintString( 0, pozY, (char*)nazwaLong, ~ANSI_WHITE);
  sprintf( str, "%s %s", __DATE__, __TIME__ );
  ssd1306_PrintString( 0, pozY+10, str, ANSI_WHITE);
  ssd1306_PrintString( 0, pozY+18, (char*)email, ANSI_WHITE);
  ssd1306_display(); // ssd1306_invertDisplay( true );


	for(u8 x=0; x<TIM_LOGO*5; x++){
		IWDG->KR = 0xaaaa;      //przeladowanie (reload) IWDG
		HAL_Delay(200);
		HAL_GPIO_TogglePin( LED_OK_GPIO_Port, LED_OK_Pin );
		if( x==TIM_LOGO/2) HAL_GPIO_WritePin( USB_PU_GPIO_Port, USB_PU_Pin, GPIO_PIN_SET );
	}


	//	#define	KK2@

	#ifdef KK2
		InitFram(); //INFO: serwisowa
	#else
		SkanujIIC();
//		#error "Wykrywanie bledu eeprom odczytu i zapisu"
	#endif

	
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */


	u16 dane[5] = { 0xaa, 0xcc };
	FLASH_write( dane, 2 );
	__NOP();


	HAL_UART_Receive_IT(&huart2, (uint8_t*)ReceivedUsart, 1);	 // Wlaczenie nasluchiwania
//	TimDemo = 20*1000;//todo:
  while (1)
  {
    IWDG->KR = 0xaaaa;      //przeladowanie (reload) IWDG
    __WFI();


    if ( !TimLedOk ) {
      TimLedOk = 500;

			if( StI2cFramWr != HAL_OK ) ReanimacjaI2C();
      HAL_GPIO_TogglePin( LED_OK_GPIO_Port, LED_OK_Pin );
    }


		u8 static prevUnused = true; ;
		if( prevUnused != FL_KeyUnused ){		// Kasuje ekran diagnostyczny
			prevUnused = FL_KeyUnused;
			TimRefreshLcd = 0;
			while( Key ) {IWDG->KR = 0xaaaa; }	// todo: niezbyt eleganckie ale tylko raz po resecie
			Key = 0;
			HAL_Delay( 50 );
			}
    u8 static Kursor = 0;
    u8 static prevKey = 0;
    if ( prevKey != Key && !prevUnused ) { // Wykrycie zmiany stanu klawiszy
      prevKey = Key;

      if ( Key & KEY_S1 ) Kursor++; if ( Kursor > CNT_EDIT_MAX ) Kursor = 0;
      if ( Key & KEY_S2 ) {
        Liczniki.cfg ^= 1 << (Kursor + CNT_EDIT_MIN);
        WriteFram();
      }
    }
    if ( TimPushKey >= 3000 && Key & KEY_S2 ) {         // Gdy dlugo naciskany S2
      Liczniki.cfg &= ~(1 << (Kursor + CNT_EDIT_MIN));  // zatrzymanie
      Liczniki.CzasPracy[Kursor + CNT_EDIT_MIN] = 0;
      TimPushKey = 0;
      WriteFram();
    }
    if ( TimPushKey >= 3000 && Key & KEY_RST1 ) {       // Gdy dlugo zwarty Rst1
      Liczniki.CzasPracy[1] = 0;
      TimPushKey = 0;
      WriteFram();
    }
    if ( TimPushKey >= 3000 && Key & KEY_RST2 ) {        // Gdy dlugo zwarty Rst2
      Liczniki.CzasPracy[2] = 0;
      TimPushKey = 0;
      WriteFram();
    }
    //todo: led RUN PWM
    //todo: RTC

u8 static FL_WR_FRAM_ERR = false;
		if ( !TimRefreshLcd  ) {
      TimRefreshLcd = 150;

      ObslugaLedtAlertow();
			if( str[STR_LEN-1] != '@' || str[STR_LEN-2] != '#' ) FL_BufStrErr=true;


      u8 static ramka = 0;
      ssd1306_clearDisplay();
      ssd1306_PrintString( 0, 0, (char*)nazwa, ~ANSI_WHITE);
      //Brak transmisji to brak wywolania "HAL_PCD_SOFCallback" w "usbd_conf.c"
      //  extern char volatile TimSof;
      //  TimSof = 50;
      if ( TimSof ) ssd1306_PrintString( SSD1306_LCDWIDTH - 3 * 6, 0, "USB", ~ANSI_WHITE);

      for (u8 x = 0; x < LICZ_LICZ && x < 6; x++) {
        float czas = Liczniki.CzasPracy[x];
        u32 min = Liczniki.CzasPracy[x] / 60;
        u32 godz = Liczniki.CzasPracy[x] / 3600 ;
        u32 dni = Liczniki.CzasPracy[x] / 86400 ;
        u8 pozY;
        pozY = 16 + x * 8;
        if ( ((ramka) >> 1) & 1 || !(Liczniki.cfg & (1 << x)) ) {
          sprintf( str, "%dd % 3d:%02d.%02d %s", (int)dni, (int)(godz % 24), (int)(min % 60), (int)(Liczniki.CzasPracy[x] % 60), Liczniki.Txt[x] );
        }
        else {
          sprintf( str, "%dd % 3d %02d %02d %s", (int)dni, (int)(godz % 24), (int)(min % 60), (int)(Liczniki.CzasPracy[x] % 60), Liczniki.Txt[x] );
        }
        ssd1306_PrintString( 8, pozY, str, ANSI_WHITE);
        if ( x == 0 ) { // Licznik globalny dodatkowo sekundy i godziny
          sprintf( str, "%0.0fs / %d'%03dh", czas, (int)(godz / 1000), (int)(godz % 1000) );
          ssd1306_PrintString( SSD1306_LCDWIDTH - strlen(str) * 6, pozY - 8, str, ANSI_WHITE);
        }
      }
      ssd1306_DrawFont( 0, 16 + Kursor * 8 + CNT_EDIT_MIN * 8, '>', ANSI_WHITE );
      ssd1306_DrawFont( SSD1306_LCDWIDTH - 8, 16 + Kursor * 8 + CNT_EDIT_MIN * 8, '<', ANSI_WHITE );


			//----- Okna alarmow ostrzezen -----//
			if( TimDemo < 10*1000 && (TimDemo % 1000 > 500 ) )		// Miganie
			{
				PrintOknoKom( "Informacja", (char*)nravt, (char*)email );
			}
			else if ( !TimDemo )
			{
				LosujCzasDemo();
				StringDemo();
				TRACK;
			}


      if ( FL_KeyUnused ) {
        if ( FL_Bod ) PrintOknoKom( (char*)txtAlarm, "Zadzia"_l"anie", "BOD" );

        if ( FL_Iwdg ) PrintOknoKom( (char*)txtAlarm, "Zadzia"_l"anie", "IWDG" );
				}


      if ( Liczniki.dziewica == DZIEWICA_INIT ) PrintOknoKom( (char*)txtAwaria, "Inicjalizacja", "konfiguracji FRAM" );
				else if ( Liczniki.dziewica == DZIEWICA_RECOVERED ) PrintOknoKom( (char*)txtOstrzezenie, "Odtworzono", "konfiguracj"_e );


			if( TimOknoTestowe ) PrintOknoKom( o_belka, o_wiersz_1, o_wiersz_2 );

      if ( TimPrtSave && (ReadTimSys() & 0x100) ) ssd1306_PrintString( 0, 8, "SAVE", ~ANSI_WHITE);
			if ( FL_WR_FRAM_ERR ) PrintOknoKom( (char*)txtAwaria, "B"_l""_s"d zapisu", "pami"_e"ci Fram" );
			if ( FL_BufStrErr ) PrintOknoKom( (char*)txtBladKrytyczny, "Uszkodzony bufor", "str[] w RAM" );
      //>>>>> Ostatni komunikat zakrywa poprzednie <<<<<


      ssd1306_display();
      ramka++;
    }


    if ( FL_WrFram ) {
      FL_WrFram = false;
      TimPrtSave = TIM_PRT_SAVE;
      if( WrFram() != CFG_OK ) FL_WR_FRAM_ERR = true; else FL_WR_FRAM_ERR = false;
      TRACK;
    }


    if ( !TimSend ) {
      TimSend = 1000;
			#define	TERM_REFRESH							15	// Maska co ile odswierzac (15=16 sek)
			u8 static refresh=TERM_REFRESH;
			++refresh;

//Liczniki.CzasPracy[0] = 0xFFFF0000;			//todo:
//Liczniki.CzasPracy[4] = 5*365*48600;			//todo:
//Liczniki.CzasPracy[5] = 0xFF000000;			//todo:
			u8 kolumna = 30, wiersz = 1;	//Esc[Line;ColumnH
	    char static txt[200];
      strcpy( str, "\x1B[s" );	// ANSI Zapamietaj pozycje kursora

			u8 static one=true;
			if( one && TimSof ){ one=false; strcat( str, "\x1B[H\x1B[2J" ); } // HOME, CLR Screen

			sprintf( txt, "\x1B[%d;%dH\x1B[42m/-------- %s -------\\", wiersz, kolumna, nazwaLong );// ANSI_BACK_GREEN		ANSI"42m"
			strcat( str, txt );
      for (u8 x = 0; x < LICZ_LICZ; x++) {
        float czas = Liczniki.CzasPracy[x];
        u32 min = Liczniki.CzasPracy[x] / 60;
        u32 godz = Liczniki.CzasPracy[x] / 3600 ;
        u32 dni = Liczniki.CzasPracy[x] / 86400 ;
        char aktywny = ' ';
        if ( Liczniki.cfg & (1 << x) ) aktywny = '*';
				//----- ramka pionowa lewa
				sprintf( txt, "\x1B[%d;%dH|\x1B[44m ", wiersz+x+1, kolumna );	// ANSI_BACK_BLUE		ANSI"44m"
				strcat( str, txt );
				//----- wypelnienie zanegowanymi spacjami (tlo)
				if( (Liczniki.cfgTerm & CFG_TERM_TLO) || !(refresh&TERM_REFRESH) ){			// Gdy wylaczone to wypelniaj co 16 sekund
					sprintf( txt, "\x1B[%d;%dH|\x1B[44m                                        ", wiersz+x+1, kolumna );
					strcat( str, txt );
					sprintf( txt, "\x1B[%d;%dH\x1B[42m|\x1B[44m ", wiersz+x+1, kolumna );	// ANSI_BACK_BLUE		ANSI"44m"
					strcat( str, txt );
					}
				//----- rekord danych
        // sprintf( txt, "[%d]%c %0.0f \t%d'%03dh \t%dd %02d:%02d:%02d \t%s \r\n", x, aktywny, czas, (int)(godz / 1000), (int)(godz % 1000), (int)dni, (int)(godz % 60), (int)(min % 60), (int)(Liczniki.CzasPracy[x] % 60), Liczniki.Txt[x] );
				sprintf( txt, "[%d]%c % 6dd % 3d:%02d:%02d %s\r\n", x, aktywny, (int)dni, (int)(godz % 24), (int)(min % 60), (int)(Liczniki.CzasPracy[x] % 60), Liczniki.Txt[x] );
        strcat( str, txt );
				//----- ramka pinowna prawa
				sprintf( txt, "\x1B[%d;%dH\x1B[42m|", wiersz+x+1, kolumna+41 );	// ANSI_BACK_GREEN		ANSI"42m"
				strcat( str, txt );
      }
			sprintf( txt, "\x1B[%d;%dH\x1B[42m\\----------------------------------------/", wiersz+LICZ_LICZ+1, kolumna );// ANSI_BACK_GREEN		ANSI"42m"
			strcat( str, txt );
      strcat( str, "\x1B[u" );	// ANSI odtworz pozycje kursora
      SendData( str, strlen(str) );
    }


    if ( ReceivedUsbDataFlag ) {
      for (u16 x = 0; x < ReceivedUsbDataFlag; x++) InterpretujCmd( ReceivedUsbData[x] );
      ReceivedUsbDataFlag = false;
    }
		int znak;
		if( (znak=GetUsart()) != 0xFFFF ) InterpretujCmd( znak );


  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */

  }
  /* USER CODE END 3 */

}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInit;

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSI|RCC_OSCILLATORTYPE_HSE
                              |RCC_OSCILLATORTYPE_LSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.LSEState = RCC_LSE_ON;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_RTC|RCC_PERIPHCLK_USB;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSE;
  PeriphClkInit.UsbClockSelection = RCC_USBCLKSOURCE_PLL_DIV1_5;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* I2C1 init function */
static void MX_I2C1_Init(void)
{

  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 1000000;
	#define I2C_FASTPLUS
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

/* IWDG init function */
static void MX_IWDG_Init(void)
{

  hiwdg.Instance = IWDG;
  hiwdg.Init.Prescaler = IWDG_PRESCALER_8;
  hiwdg.Init.Reload = 4095;
  if (HAL_IWDG_Init(&hiwdg) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

/* RTC init function */
static void MX_RTC_Init(void)
{

  RTC_TimeTypeDef sTime;
  RTC_DateTypeDef DateToUpdate;

    /**Initialize RTC Only 
    */
  hrtc.Instance = RTC;
  if(HAL_RTCEx_BKUPRead(&hrtc, RTC_BKP_DR1) != 0x32F2){
  hrtc.Init.AsynchPrediv = RTC_AUTO_1_SECOND;
  hrtc.Init.OutPut = RTC_OUTPUTSOURCE_ALARM;
  if (HAL_RTC_Init(&hrtc) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Initialize RTC and set the Time and Date 
    */
  sTime.Hours = 0x1;
  sTime.Minutes = 0x0;
  sTime.Seconds = 0x0;

  if (HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BCD) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  DateToUpdate.WeekDay = RTC_WEEKDAY_MONDAY;
  DateToUpdate.Month = RTC_MONTH_JULY;
  DateToUpdate.Date = 0x1;
  DateToUpdate.Year = 0x18;

  if (HAL_RTC_SetDate(&hrtc, &DateToUpdate, RTC_FORMAT_BCD) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    HAL_RTCEx_BKUPWrite(&hrtc,RTC_BKP_DR1,0x32F2);
  }

}

/* SPI1 init function */
static void MX_SPI1_Init(void)
{

  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi1.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_8;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

/* TIM3 init function */
static void MX_TIM3_Init(void)
{

  TIM_ClockConfigTypeDef sClockSourceConfig;
  TIM_MasterConfigTypeDef sMasterConfig;
  TIM_OC_InitTypeDef sConfigOC;

  htim3.Instance = TIM3;
  htim3.Init.Prescaler = 4999;
  htim3.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim3.Init.Period = 100;
  htim3.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim3.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
  if (HAL_TIM_Base_Init(&htim3) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim3, &sClockSourceConfig) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  if (HAL_TIM_PWM_Init(&htim3) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim3, &sMasterConfig) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  sConfigOC.OCMode = TIM_OCMODE_PWM1;
  sConfigOC.Pulse = 50;
  sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCFastMode = TIM_OCFAST_ENABLE;
  if (HAL_TIM_PWM_ConfigChannel(&htim3, &sConfigOC, TIM_CHANNEL_2) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

  HAL_TIM_MspPostInit(&htim3);

}

/* USART2 init function */
static void MX_USART2_UART_Init(void)
{

  huart2.Instance = USART2;
  huart2.Init.BaudRate = 9600;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

}

/** Configure pins as 
        * Analog 
        * Input 
        * Output
        * EVENT_OUT
        * EXTI
*/



static void MX_GPIO_Init(void)
{

  GPIO_InitTypeDef GPIO_InitStruct;

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, LCD_CS_Pin|LCD_C_D_Pin|LED_ERR_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(USB_PU_GPIO_Port, USB_PU_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LED_ALERT_GPIO_Port, LED_ALERT_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(LED_OK_GPIO_Port, LED_OK_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pins : Key1_Pin Key2_Pin */
  GPIO_InitStruct.Pin = Key1_Pin|Key2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : LCD_CS_Pin LCD_C_D_Pin USB_PU_Pin LED_ERR_Pin */
  GPIO_InitStruct.Pin = LCD_CS_Pin|LCD_C_D_Pin|USB_PU_Pin|LED_ERR_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : EN1_Pin EN2_Pin BOOT1_Pin Rst1_Pin 
                           Rst2_Pin */
  GPIO_InitStruct.Pin = EN1_Pin|EN2_Pin|BOOT1_Pin|Rst1_Pin 
                          |Rst2_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : LED_ALERT_Pin LED_OK_Pin */
  GPIO_InitStruct.Pin = LED_ALERT_Pin|LED_OK_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  file: The file name as string.
  * @param  line: The line in file as a number.
  * @retval None
  */
void _Error_Handler(char *file, int line)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while (1)
  {
    PrintError();
  }
#ifndef I2C_FASTPLUS
#warning "Zwieksz predkosc I2C do 1Mb/s - hi2c1.Init.ClockSpeed = "
#warning "Dodaj kontrole pakietow SOF - TimSof"	
#endif
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  PrintError();
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
